
sub GetProcesses {
# Adapted from a script by Jutta M. Klebe (Author of PerfLib module)
# shows the task list like the task manager

	# returns the task list as a hash %list:
	# $list{'task_report'} - a formatted display of tasks
	# $list{'task_list'} - array of task names
	# $list{'idle_time'} - percentage of cpu time taken by the 'Idle process'
	# $list{'threads'} - total number of process threads
	
	use Win32::Perflib;   
	
	my ($server) = @_[0];
	my %counter;
	my %process; # task list
	my %list;  # the formatted task list that will be returned from the function
	
	Win32::PerfLib::GetCounterNames($server, \%counter);
	my %r_counter = map { $counter{$_} => $_ } keys %counter;

	# Find the counter ID's
	my $process_obj = $r_counter{'Process'}; # 230
	my $process_id = $r_counter{'ID Process'}; # 784
	my $processor_time = $r_counter{'% Processor Time'}; # 6
	my $elapsed = $r_counter {'Elapsed Time'}; # 684
	my $memory =$r_counter {'Working Set'}; # 180
	my $page_faults = $r_counter {'Page Faults/sec'}; # 28
	my $virtual_memory = $r_counter {'Private Bytes'}; # 186
	my $priority = $r_counter {'Priority Base'}; # 682
	my $threads = $r_counter {'Thread Count'}; # 680
	
	# Don't need these huge hashes anymore
	undef (%counter);
	undef (%r_counter);

	# Retrieve the information from the server into two objects
	# over a 5 second interval
	my $perflib = new Win32::PerfLib($server);
	my $proc_ref0 = {};
	my $proc_ref1 = {};
	$perflib->GetObjectList($process_obj, $proc_ref0);
	sleep 10; # Get data over a 10 second interval
	$perflib->GetObjectList($process_obj, $proc_ref1);
	$perflib->Close();

	# Two instances of perflib data
	my $instance_ref0 = $proc_ref0->{Objects}->{$process_obj}->{Instances};
	my $instance_ref1 = $proc_ref1->{Objects}->{$process_obj}->{Instances};

	# Process information hashes, indexed by process id
	my (%id, %cputime, %priority, %page_faults, %virtual_memory, %threads);

	foreach $p (keys %{$instance_ref0}){
		my $counter_ref0 = $instance_ref0->{$p}->{Counters};
		my $counter_ref1 = $instance_ref1->{$p}->{Counters};
		foreach $i (keys %{$counter_ref0})    {
			next if ($instance_ref0->{$p}->{Name} eq "_Total");
			
			if($counter_ref0->{$i}->{CounterNameTitleIndex} == $process_id) {
				$process{ $counter_ref0->{$i}->{Counter} } =
					$instance_ref0->{$p}->{Name};
				
				$id{ $counter_ref0->{$i}->{Counter} } = $p;
		
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $processor_time) {
				$Numerator0 = $counter_ref0->{$i}->{Counter};
				$Denominator0 = $proc_ref0->{PerfTime100nSec};
				$Numerator1 = $counter_ref1->{$i}->{Counter};
				$Denominator1 = $proc_ref1->{PerfTime100nSec};
				$proc_time{$p} = ($Numerator1 - $Numerator0) /
				  ($Denominator1 - $Denominator0 ) * 100;
				$cputime{$p} = int($counter_ref1->{$i}->{Counter} / 10000000);
			
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $memory) {
				$memory{$p} = int($counter_ref0->{$i}->{Counter} / 1024);
			
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $page_faults) {
				$page_faults{$p} = $counter_ref1->{$i}->{Counter};        
			
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $virtual_memory) {
				$virtual_memory{$p} = int($counter_ref0->{$i}->{Counter} / 1024);
			
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $priority) {
				$priority{$p} = $counter_ref0->{$i}->{Counter};        
				
			} elsif($counter_ref0->{$i}->{CounterNameTitleIndex} == $threads) {
				$threads{$p} = $counter_ref0->{$i}->{Counter};        
			}    
		}
	}
	
	
	$list{'task_report'} .= " PID Process         CPU   CPU-Time     Memory       PF   Virt.Mem Priority Thr\n";

	foreach $p (sort { $a <=> $b } keys %process) {   
		my $id = $id{$p};
		my $seconds = $cputime{$id};    
		my $hour = int($seconds / 3600);
		$seconds -= $hour * 3600;    
		my $minute = int($seconds / 60);
		$seconds -= $minute * 60;    
		
		my $prio;
		
		if ($priority{$id} > 15) {
			$prio = "Realtime";    
		} elsif ($priority{$id} > 10 ) { 
			$prio = "High";    
		} elsif ($priority{$id} > 5 ) {
			$prio = "Normal";    
		} elsif ($priority{$id} > 0 ) {
			$prio = "Low";    
		} else {
			$prio = "Unknown";    
		}

		$list{'task_report'} .= sprintf("% 4d %-14s%5.2f  % 3d:%02d:%02d % 8d K % 8d % 8d K %8s % 3d\n",
			   $p, $process{$p}, $proc_time{$id}, $hour, $minute, $seconds,
			   $memory{$id}, $page_faults{$id}, $virtual_memory{$id},
			   $prio, $threads{$id});
	}
	
	my @procs = values (%process);
	$list{'task_list'} = [ @procs ];
	
	# The percentage of time taken up by the idle process can be used to calculate cpu load (subtract from 100 :)	
	$list{'idle_time'} = sprintf ("%.2f", $proc_time{$id{0}});

	return %list;
	
} # End sub GetProcesses 

1;
# I am an included Perl file, so I need this.
