# NT version
# This module encapsulated the status function. It sends information to the
# Spong server. 
#
# It reports the current status of a service, and sends along
# a string of information that might be helpful in diagnosing the problem.
# This code is modeled after the bb program, but is a little different in
# that it handles multi-line messages and send over the time as an int,
# rather then a string.
#
# Modified by Joanna Ziembicki on July 24, 2000 to fix the core dump 
# if spong-nt-client can't connect to spong-server
#
# Jan. 18, 2001: changed @ISA = qw(Exporter Spong::Status); to 
#                        @ISA = qw(Exporter); to fix memory leak
#
# methods
#  status( SERVERADDR, HOST, SERVICE, COLOR, SUMMARY, MESSAGE )


package Spong::Status;

require Exporter;

use strict;
use vars qw(@ISA @EXPORT_OK $VERSION);
use Carp;
use IO::Socket;

@ISA = qw(Exporter);
@EXPORT_OK = qw(status);
$VERSION = 0.02;


sub status {
   my( $addr, $host, $cat, $color, $summary, $message ) = @_;
   my( $sock, $ok );

   # Try 30 times to connect to spong-server
   for ( 1..30 ) {
      $sock = IO::Socket::INET->new( PeerAddr => $addr,
                                     PeerPort => $main::SPONG_UPDATE_PORT,
                                     Proto    => 'tcp',
                                     Timeout  => 30,
                                     Reuse    => 1,
                                   );

      if ( defined $sock ) { # Once we got a connection, exit the loop
         last;
      }
      sleep 1;  # Pause a second to not overwhelm the network stack
   }

   if (! defined $sock ) {
     # print an error message to standard output and logfile..
     # This is an icky workaround, since it prints to standard output (sin!) 
     # (and is only noticeable in debug mode)
     # but it is the least messy way I could find..so far..
     
     print scalar localtime(), " Error: Could not connect with Spong Server: $@\n";
      
   } else {
	  # it would be nice if 'alarm' were implemented in NT... 
	  # if spong-server dies suddenly here, there could be problems 
	  
      $sock->autoflush(1);
      $sock->print("status $host $cat $color " . time(). " $summary\n");
      $sock->print("$message\n");

      undef $sock;
      $ok = 1;
   }
   # print STDERR scalar localtime, " can't connect to spong server.\n" if ! $ok;
}



